package com.persagy.adm.server.algorithm.controller;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import com.persagy.adm.server.algorithm.constant.AdmServerConstant;
import com.persagy.adm.server.algorithm.domain.ModelFileVo;
import com.persagy.adm.server.algorithm.domain.QueryModelFileVo;
import com.persagy.adm.server.algorithm.entity.AdmModelFile;
import com.persagy.adm.server.algorithm.service.AdmModelService;
import com.persagy.dmp.common.context.AppContext;
import com.persagy.dmp.common.model.response.CommonResult;
import com.persagy.dmp.common.utils.ResultHelper;
import com.persagy.dmp.digital.entity.ObjectDigital;
import lombok.RequiredArgsConstructor;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 01-模型文件管理
 * 后端服务名:adm-server
 * @author : lijie
 * Update By 2022/1/12 14:01
 */
@RestController
@RequestMapping("/model")
@RequiredArgsConstructor
public class AdmModelController {

    private final AdmModelService admModelService;

    /**
     * 010101-模型文件-根据建筑id查询模型文件列表
     * @param queryModelFileVo : 模型文件查询信息
     * @return : com.persagy.dmp.common.model.response.CommonResult<java.util.List<com.persagy.adm.server.algorithm.entity.AdmModelFile>>
     * @author : lijie
     * Update By 2022/1/12 16:32
     * @undone
     */
    @PostMapping(value = "/queryModelFilesByBuildingId")
    public CommonResult<List<ModelFileVo>> queryFloorsByBuildingId(@RequestBody QueryModelFileVo queryModelFileVo) {
        // 1.根据建筑id查询建筑下的所有楼层
        List<ObjectDigital> allFloors = admModelService.queryFloorsByBuildingId(queryModelFileVo.getBuildingId());
        if (CollUtil.isEmpty(allFloors)){
            return ResultHelper.multi(new ArrayList<>());
        }
        // 2.根据floorId查询最新的模型文件信息
        Map<String, AdmModelFile> modelFileMap = admModelService
                .queryCurrentModelFileByFloorIds(allFloors.stream().map(ObjectDigital::getId).collect(Collectors.toList()));
        // 3.组装返回值
        List<ModelFileVo> result = new ArrayList<>(allFloors.size());
        for (ObjectDigital floor : allFloors) {
            ModelFileVo vo = new ModelFileVo();
            result.add(vo);
            vo.setFloorId(floor.getId());
            vo.setBuildingId(queryModelFileVo.getBuildingId());
            vo.setFloorName(floor.getLocalName());
            if (null!=floor.getInfos()
                    && floor.getInfos().has(AdmServerConstant.FLOOR_SEQUENCE_ID)
                    && floor.getInfos().get(AdmServerConstant.FLOOR_SEQUENCE_ID).isIntegralNumber()){
                vo.setFloorSequenceId(floor.getInfos().get(AdmServerConstant.FLOOR_SEQUENCE_ID).asInt());
            }
            AdmModelFile admModelFile = modelFileMap.get(floor.getId());
            if (null==admModelFile){
                continue;
            }
            vo.setDataFileId(admModelFile.getDataFileId());
            vo.setMinDataFileId(admModelFile.getMinDataFileId());
            vo.setOriginalName(admModelFile.getOriginalName());
            vo.setRevitFileId(admModelFile.getRevitFileId());
            vo.setRevitVersion(admModelFile.getRevitVersion());
            vo.setStatus(admModelFile.getStatus());
            vo.setUserId(admModelFile.getUserId());
            vo.setUserName(admModelFile.getUserName());
        }
        return ResultHelper.multi(result);
    }
    /**
     * 010102-模型文件-创建模型文件信息
     * @param modelFileVo : 模型文件信息
     * @return : com.persagy.dmp.common.model.response.CommonResult<java.lang.Void>
     * @author : lijie
     * Update By 2022/1/12 16:54
     * @undone
     */
    @PostMapping(value = "/saveModelFile")
    public CommonResult<List<Void>> saveModelFile(@Validated @RequestBody ModelFileVo modelFileVo) {
        Map<String, AdmModelFile> modelFileMap = admModelService.queryCurrentModelFileByFloorIds(CollUtil.newArrayList(modelFileVo.getFloorId()));
        AdmModelFile admModelFile = modelFileMap.get(modelFileVo.getFloorId());
        AdmModelFile modelFile = BeanUtil.copyProperties(modelFileVo, AdmModelFile.class);
        modelFile.setGroupCode(AppContext.getContext().getGroupCode());
        modelFile.setProjectId(AppContext.getContext().getProjectId());
        modelFile.setCreator(AppContext.getContext().getAccountId());
        modelFile.setModifier(AppContext.getContext().getAccountId());
        admModelService.saveModelFile(modelFile);

        return ResultHelper.multi(new ArrayList<>());
    }

}
